package vsl;

import java.util.*;

import org.w3c.dom.*;
import vsl.dataflow.*;
import vsl.lang.*;

/*
 * Klasse fr ein visuelles Fragment-Programm
 */
public class FragmentProgram extends DataFlowProgram
{

  /*
   * Erzeugt ein Fragment-Programm aus einem DOM-Dokument.
   */
  public FragmentProgram( Document document )
  {
    super( document );
  }

  /*
   * Erzeugt ein neues Fragment-Programm.
   */
  public FragmentProgram( String name )
  {
    super( name,  Struct.getStruct( "f_in_ARB_small" ), Struct.getStruct( "f_out_ARB" ) );
  }

  /*
   * Erzeugt ein neues Fragment-Programm unter Angabe der Ein- und Ausgabe-Structs.
   */
  public FragmentProgram( String name, Struct inputStruct, Struct outputStruct )
  {
    super( name, inputStruct, outputStruct);
  }

  /*
   * Exportiert das Programm in ein DOM-Document.
   */
  public Element exportXML( Document document )
  {
    Element rootXML = document.createElement( "FragmentProgram" );
    rootXML.setAttribute( "name", getName() );

    Element inputStructXML = document.createElement( "InputStruct" );
    inputStructXML.setAttribute( "name", inputStructInstanceName);
    inputStructXML.setAttribute( "type", inputStruct.getName());
    rootXML.appendChild( inputStructXML );

    for ( int i = 0; i < exports.getAnchorCount(); i++ )
    {
      DataFlowDestinationAnchor export = ( DataFlowDestinationAnchor ) exports.getAnchor( i );
      Element outputXML = document.createElement( "Output" );
      outputXML.setAttribute( "name", export.getName() );
      outputXML.setAttribute( "type", export.getType().toString() );
      rootXML.appendChild( outputXML );
    }
    for ( int i = 0; i < imports.getAnchorCount(); i++ )
    {
      DataFlowSourceAnchor imp = ( DataFlowSourceAnchor ) imports.getAnchor( i );
      Element inputXML = document.createElement( "Input" );
      inputXML.setAttribute( "name", imp.getName() );
      inputXML.setAttribute( "type", imp.getType().toString() );
      if( imp.getType().getJavaType() != DataType.J_UNDEFINED )
      {
        inputXML.setAttribute( "default", imp.getDefault().toString() );
      }
      rootXML.appendChild( inputXML );
    }

    Element outputStructXML = document.createElement( "OutputStruct" );
    outputStructXML.setAttribute( "name", outputStructInstanceName);
    outputStructXML.setAttribute( "type", outputStruct.getName());
    rootXML.appendChild( outputStructXML );

    for ( int i = 0; i < outputStructExportCount; i++ )
    {
      Element exportXML = document.createElement( "Export" );
      DataFlowDestinationAnchor export = ( DataFlowDestinationAnchor ) outputStructExports.getAnchor( i );
      exportXML.setAttribute( "name", export.getName() );
      if ( export.isConnected() )
      {
        Element connectionXML = export.getEdge().exportXML( document );
        exportXML.appendChild( connectionXML );
        rootXML.appendChild( exportXML );
      }
    }

    for ( int i = 0; i < exports.getAnchorCount(); i++ )
    {
      Element exportXML = document.createElement( "Export" );
      DataFlowDestinationAnchor export = ( DataFlowDestinationAnchor ) exports.getAnchor( i );
      exportXML.setAttribute( "name", export.getName() );
      if ( export.isConnected() )
      {
        Element connectionXML = export.getEdge().exportXML( document );
        exportXML.appendChild( connectionXML );
        rootXML.appendChild( exportXML );
      }
    }
    ArrayList sortedNodes = nodes.getSortedList();
    for ( int i = 0; i < sortedNodes.size(); i++ )
    {
      DataFlowNode node = ( DataFlowNode ) sortedNodes.get( i );
      if ( node.getMaxDistance() != -1 )
      {
        Element componentXML = node.exportXML( document );
        rootXML.appendChild( componentXML );
      }
    }

    return rootXML;
  }

}
