package vsl.dataflow;

import java.awt.*;
import java.awt.image.*;

import java.io.*;

import javax.imageio.ImageIO;

import vsl.lang.*;
import vsl.dataflow.util.*;

import org.w3c.dom.*;

/*
 * Knoten, der verschiedene Komponenten zu einem Vektor zusammenfhrt.
 */
public class DataFlowConstructorNode extends DataFlowNode
{

  Constructor constructor;

  /*
   * Bilder zur Darstellung des Knoten
   */
  protected static BufferedImage image;
  protected BufferedImage icon;

  /*
   * Laden des Knoten-Bildes
   */
  static
  {
    try
    {
      image = ImageIO.read( new File( "images/block_small.png" ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }
  }

  /*
   * Konstruktor
   */
  public DataFlowConstructorNode( Element element )
  {
    constructor = Constructor.getConstructor(element.getAttribute("name"));
    initConstructorNode();
  }


  public DataFlowConstructorNode( Constructor constructor )
  {
    this.constructor = constructor;
    initConstructorNode();
  }

  public void initConstructorNode()
  {
    name = constructor.getName();
    this.id = "00";
    try
    {
      icon = ImageIO.read( new File( constructor.getIconFilename()) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }

    inputs = new DataFlowAnchorList();
    for ( int i = 0; i < constructor.getInputCount(); i++ )
    {
      DataType dataType = constructor.getInputType( i );
      DataFlowDestinationAnchor anchor = new DataFlowDestinationAnchor(
          constructor.getInputName( i ),
          dataType, dataType.getDefaultValue(), this );
      inputs.addAnchor( anchor );
    }

    outputs = new DataFlowAnchorList();
    DataFlowSourceAnchor anchor = new DataFlowSourceAnchor( "v", constructor.getType(), this );
    outputs.addAnchor( anchor );
    showLabels();
    setSize( image.getWidth(), image.getHeight() );
  }

  public void paint( Graphics g )
  {
    g.drawImage( image, 0, 0, getWidth(), getHeight(), this );
    g.drawImage( icon, (getWidth()-icon.getWidth())/2, (getHeight()-icon.getHeight())/2, icon.getWidth(), icon.getHeight(), this );
  }

  public Element exportXML( Document document )
  {
    Element root = document.createElement( "Node" );
    root.setAttribute( "type", "constructor" );
    root.setAttribute( "name", getName() );
    root.setAttribute( "result", constructor.getType().toString());
    root.setAttribute( "id", getId() );
    root.setAttribute( "x", Integer.toString( getX() ) );
    root.setAttribute( "y", Integer.toString( getY() ) );
    for ( int i = 0; i < inputs.getAnchorCount(); i++ )
    {
      DataFlowDestinationAnchor input = ( DataFlowDestinationAnchor ) inputs.getAnchor( i );
      Element dataElement = document.createElement( "Data" );
      dataElement.setAttribute( "name", input.getName() );
      root.appendChild( dataElement );
      if ( input.isConnected() )
      {
        DataFlowEdge edge = input.getEdge();
        Element inputElement = edge.exportXML( document );
        dataElement.appendChild( inputElement );
      }
      else
      {
        Element defaultElement = document.createElement( "Default" );
        dataElement.appendChild( defaultElement );
        if( input.getType().hasDefaultValue() )
        {
          dataElement.setAttribute( "value", input.getType().getDefaultValue().toString() );
        }
      }
    }
    return root;
  }

  public int getNodeType()
  {
    return DF_CONSTRUCTOR_NODE;
  }

}