package vsl.dataflow;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.geom.Rectangle2D;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;

import javax.imageio.ImageIO;

import org.w3c.dom.*;

import vsl.lang.DataType;
import vsl.lang.Function;
import vsl.lang.Parameter;
import vsl.dataflow.util.*;

public class DataFlowFunctionNode extends DataFlowNode
{

  Function function;

  protected static BufferedImage image;
  protected BufferedImage icon;

  static
  {
    try
    {
      image = ImageIO.read( new File( "images/block_small.png" ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }
  }

  public DataFlowFunctionNode( Element element )
  {
    function = Function.getFunction( element.getAttribute( "key" ) );
    functionNodeInit();
  }

  public DataFlowFunctionNode( Function function )
  {
    this.function = function;
    functionNodeInit();
  }

  public void functionNodeInit()
  {
    this.id = "00";
    name = function.getName();
    icon = function.getIcon();

    inputs = new DataFlowAnchorList();
    for ( int i = 0; i < function.getParameterCount(); i++ )
    {
      Parameter parameter = function.getParameter( i );
      DataType inputType = parameter.getType();
      String inputName = parameter.getName();
      DataFlowDestinationAnchor anchor = new DataFlowDestinationAnchor(
          inputName, inputType, this );
      inputs.addAnchor( anchor );
    }

    outputs = new DataFlowAnchorList( 1 );
    DataFlowSourceAnchor anchor = new DataFlowSourceAnchor( function.getName(),
        function.getReturnType(), this );
    outputs.setAnchor( 0, anchor );
    showLabels();

    setSize( image.getWidth(), image.getHeight() );
  }

  public void paint( Graphics g )
  {
    g.drawImage( image, 0, 0, getWidth(), getHeight(), this );
    g.drawImage( icon, ( getWidth() - icon.getWidth() ) / 2,
                 ( getHeight() - icon.getHeight() ) / 2, icon.getWidth(), icon.getHeight(), this );
    Font font = new Font( "SansSerif", Font.PLAIN, 10 );
    g.setFont( font );
    String title = name;
    Rectangle2D rectangle = g.getFontMetrics( font ).getStringBounds( title, g );
    int fontHeight = ( int ) rectangle.getHeight();
    g.drawString( title, 5, fontHeight );
  }

  // exports itself as DOM element
  public Element exportXML( Document document )
  {
    Element root = document.createElement( "Node" );
    root.setAttribute( "type", "function" );
    root.setAttribute( "name", getName() );
    root.setAttribute( "result", function.getReturnType().toString() );
    root.setAttribute( "key", function.key );
    root.setAttribute( "id", getId() );
    root.setAttribute( "x", Integer.toString( getX() ) );
    root.setAttribute( "y", Integer.toString( getY() ) );
    for ( int i = 0; i < inputs.getAnchorCount(); i++ )
    {
      DataFlowDestinationAnchor input = ( DataFlowDestinationAnchor ) inputs.getAnchor( i );
      Element dataElement = document.createElement( "Data" );
      dataElement.setAttribute( "name", input.getName() );
      root.appendChild( dataElement );
      if ( input.isConnected() )
      {
        DataFlowEdge edge = input.getEdge();
        Element inputElement = edge.exportXML( document );
        dataElement.appendChild( inputElement );
      }
      else
      {
        Element defaultElement = document.createElement( "Default" );
        dataElement.appendChild( defaultElement );
        if ( input.getType().hasDefaultValue() )
        {
          dataElement.setAttribute( "value", input.getType().getDefaultValue().toString() );
        }
      }
    }
    return root;
  }

  public int getNodeType()
  {
    return DF_FUNCTION_NODE;
  }

}