package vsl.dataflow;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import java.io.IOException;
import java.io.File;

import javax.imageio.ImageIO;

import org.w3c.dom.*;

import vsl.lang.DataType;
import vsl.lang.Operator;
import vsl.dataflow.util.DataFlowAnchorList;

/*
 * Klasse fr Operator-Knoten.
 */
public class DataFlowOperatorNode extends DataFlowNode
{

  protected static BufferedImage image;
  protected BufferedImage icon;

  static
  {
    try
    {
      image = ImageIO.read( new File( "images/block_small.png" ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }
  }

  Operator operator;

  public DataFlowOperatorNode( Element element )
  {
    operator = Operator.getOperator( element.getAttribute("key") );
    init();
  }

  public DataFlowOperatorNode( Operator operator )
  {
    this.operator = operator;
    init();
  }

  public void init()
  {
    this.id = "00";
    this.icon = operator.getIcon();

    name = operator.getName();

    int inputCount;
    if ( operator.getOperand2() == null )
    {
      inputCount = 1;
    }
    else
    {
      inputCount = 2;
    }
    inputs = new DataFlowAnchorList( inputCount );

    DataFlowDestinationAnchor anchor1 = new DataFlowDestinationAnchor(
        "op1", operator.getOperand1(), this );
    inputs.addAnchor( anchor1 );
    if ( inputCount == 2 )
    {
      DataFlowDestinationAnchor anchor2 = new DataFlowDestinationAnchor(
          "op2", operator.getOperand2(), this );
      inputs.addAnchor( anchor2 );
    }

    outputs = new DataFlowAnchorList( 1 );
    DataFlowSourceAnchor anchor = new DataFlowSourceAnchor( "res",
        operator.getResult(), this );
    outputs.setAnchor( 0, anchor );
    hideLabels();
    setSize( image.getWidth(), image.getHeight() );
  }

  public void paint( Graphics g )
  {
    g.drawImage( image, 0, 0, getWidth(), getHeight(), this );
    g.drawImage( icon, ( getWidth() - icon.getWidth() ) / 2,
                             ( getHeight() - icon.getHeight() ) / 2, icon.getWidth(), icon.getHeight(), this );
  }

  // exports itself as DOM Element
  public Element exportXML( Document document )
  {
    Element root = document.createElement( "Node" );
    root.setAttribute( "type", "operator" );
    root.setAttribute( "name", getName() );
    root.setAttribute( "result", operator.getResult().toString() );
    root.setAttribute( "key", operator.getKey() );
    root.setAttribute( "id", getId() );
    root.setAttribute( "symbol", operator.getSymbol() );
    root.setAttribute( "x", Integer.toString( getX() ) );
    root.setAttribute( "y", Integer.toString( getY() ) );

    switch ( operator.getOperatorType() )
    {
      case Operator.INFIX:
        root.setAttribute( "op_type", "infix" );
        break;
      case Operator.PRAEFIX:
        root.setAttribute( "op_type", "praefix" );
        break;
      case Operator.POSTFIX:
        root.setAttribute( "op_type", "postfix" );
        break;
    }

    for ( int i = 0; i < inputs.getAnchorCount(); i++ )
    {
      DataFlowDestinationAnchor input = ( DataFlowDestinationAnchor ) inputs.getAnchor( i );
      Element dataElement = document.createElement( "Data" );
      dataElement.setAttribute( "name", input.getName() );
      root.appendChild( dataElement );
      if ( input.isConnected() )
      {
        DataFlowEdge edge = input.getEdge();
        Element inputElement = edge.exportXML( document );
        dataElement.appendChild( inputElement );
      }
      else
      {
        Element defaultElement = document.createElement( "Default" );
        dataElement.appendChild( defaultElement );
        if( input.getType().hasDefaultValue() )
        {
          dataElement.setAttribute( "value", input.getType().getDefaultValue().toString() );
        }
      }
    }
    return root;
  }

  public  int getNodeType()
  {
    return DF_OPERATOR_NODE;
  }

}