package vsl.gui;

import java.text.*;

import java.awt.*;
import javax.swing.*;

import vsl.lang.*;

/*
 * GUI-Element, um einen Datenwert zu editieren.
 */
public class DataValueEditPanel extends JPanel
{
  /*
   * Typ des Datenwerts.
   */
  DataType type;

  JFormattedTextField floatTextFields[];
  JPanel components[];
  JPanel rows[];
  int textFieldCount = 0;

  JPanel contentPane;
  JPanel editPane;

  DecimalFormatSymbols decSymbols;

  public DataValueEditPanel( DataType type )
  {
    this.type = type;
    contentPane = new JPanel();
    add( contentPane );

    editPane = new JPanel();
    contentPane.add( editPane );

    decSymbols = new DecimalFormatSymbols();
    decSymbols.setDecimalSeparator( '.' );
    floatTextFields = new JFormattedTextField[16];
    for ( int i = 0; i < 16; i++ )
    {
      floatTextFields[i] = new JFormattedTextField( new DecimalFormat( "0.000", decSymbols ) );
      floatTextFields[i].setColumns( 4 );
    }

    rows = new JPanel[4];
    for ( int i = 0; i < 4; i++ )
    {
      rows[i] = new JPanel();
    }
    setPreferredSize( new Dimension( 200, 300 ) );
    update();
  }

  /*
   * Setzt den Datentyp.
   */
  public void setType( DataType type )
  {
    this.type = type;
    editPane.removeAll();
    update();
  }

  /*
   * ndert das Editier-Formular, falls Datentyp wechselt.
   */
  public void update()
  {
    contentPane.removeAll();

    Format format = null;
    switch ( type.getJavaType() )
    {
      case DataType.J_FLOAT:
        format = new DecimalFormat( "0.000", decSymbols );
        break;
    }

    if ( type.getTypeClass() == DataType.ELEMENTAL )
    {
      rows[0].removeAll();
      if( type.getJavaType() == type.J_FLOAT )
      {
        DataValue value = type.getDefaultValue();
        rows[0].add( floatTextFields[0] );
        floatTextFields[0].setText( value.getValueAsString(0));
        floatTextFields[0].setVisible( true );
        editPane.add( rows[0] );
      }
    }
    else if ( type.getTypeClass() == DataType.VECTOR )
    {
      DataValue value = type.getDefaultValue();
      editPane.setLayout( new BorderLayout() );
      rows[0].removeAll();
      rows[0].add( new JLabel( type.toString() + "(" ) );
      for ( int i = 0; i < type.rows; i++ )
      {
        rows[0].add( floatTextFields[i] );
        floatTextFields[i].setText( value.getValueAsString(i));
        if ( i == type.rows - 1 )
        {
          rows[0].add( new JLabel( ")" ) );
        }
        else
        {
          rows[0].add( new JLabel( "," ) );
        }
      }
      editPane.add( rows[0] );

    }
    else if ( type.getTypeClass() == DataType.MATRIX )
    {
      DataValue value = type.getDefaultValue();
      editPane.setLayout( new GridLayout( type.rows + 1, 1 ) );
      textFieldCount = type.rows * type.columns;

      editPane.add( new JLabel( type.toString() + "(" ) );

      for ( int i = 0; i < type.rows; i++ )
      {
        rows[i].removeAll();
        for ( int j = 0; j < type.columns; j++ )
        {
          floatTextFields[i * type.columns + j ].setText( value.getValueAsString(i * type.columns + j ));
          rows[i].add( floatTextFields[i * type.columns + j] );
          if ( i == type.rows - 1 && j == type.columns - 1 )
          {
            rows[i].add( new JLabel( ")" ) );
          }
          else
          {
            rows[i].add( new JLabel( "," ) );
          }
        }
        editPane.add( rows[i] );
      }
    }
    contentPane.add( editPane );
    contentPane.repaint();
  }

  /*
   * Gibt den Datenwert zurck.
   */
  public DataValue getValue()
  {
    DataValue value = new DataValue( type );
    switch( type.getJavaType() )
    {
      case DataType.J_FLOAT:
      int n = type.rows* type.columns;
      float[] data = new float[n];
      for( int i=0; i< n; i++ )
      {
        data[i] = Float.parseFloat( floatTextFields[i].getText());
      }
      value.setData( data );
      break;
    }
    return value;
  }

}