package vsl.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import vsl.*;
import vsl.dataflow.*;

public class DiagramFrame extends JInternalFrame
{
  JTabbedPane tabbedPane;

  public DiagramFrame()
  {
    super( "Diagram Editor", true );
    initFrame();
  }

  public void initFrame()
  {
    tabbedPane = new JTabbedPane();
    tabbedPane.setPreferredSize( new Dimension( 800, 600 ) );
    setContentPane( tabbedPane );

    setVisible( true );
    pack();
  }

  public void addVertexProgram( VertexProgram vprogram )
  {
    DataFlowCanvas vertexCanvas = new DataFlowCanvas( vprogram );
    tabbedPane.add( vertexCanvas, "Vertex Program: " + vprogram.getName() );
    tabbedPane.setSelectedComponent(vertexCanvas);
  }

  public void addFragmentProgram( FragmentProgram fprogram )
  {
    DataFlowCanvas fragmentCanvas = new DataFlowCanvas( fprogram );
    tabbedPane.add( fragmentCanvas, "Fragment Program: " + fprogram.getName() );
    tabbedPane.setSelectedComponent(fragmentCanvas);
  }

  public void addDiagram( DataFlowDiagram diagram )
  {
    DataFlowCanvas canvas = new DataFlowCanvas( diagram );
    tabbedPane.add( canvas, "Diagram: " + diagram.getName() );
    tabbedPane.setSelectedComponent(canvas);
  }

  public void removeSelected()
  {
    remove( tabbedPane.getSelectedComponent() );
  }

  public void renameSelected( String name )
  {
    ((DataFlowCanvas)tabbedPane.getSelectedComponent()).getDiagram().setName(name);
  }

  public DataFlowCanvas getCanvas()
  {
    return ( DataFlowCanvas ) tabbedPane.getSelectedComponent();
  }

}