package vsl.lang;

import java.util.*;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


public class Constructor
{

  public static Vector constructors;
  public static HashMap constructorsHashMap;

  static
  {
    constructors = new Vector();
    constructorsHashMap = new HashMap();
    importConstructors( "xml/constructors.xml" );
  }

  public static Vector getConstructors()
  {
    return constructors;
  }


  public static void addConstructor( Constructor c )
  {
    constructors.add( c );
    constructorsHashMap.put(c.getName(), c);
  }

  public static void importConstructors( String filename )
  {
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating( true );

      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler( new DefaultHandler() );

      Document document = builder.parse( filename );

      Element rootElement = document.getDocumentElement();

      NodeList constructorListXML = rootElement.getElementsByTagName( "Constructor" );
      for ( int i = 0; i < constructorListXML.getLength(); i++ )
      {
        Element constructorXML = ( Element ) ( constructorListXML.item( i ) );
        String constructorName = constructorXML.getAttribute("name");
        DataType constructorType = DataType.getDataType( constructorXML.getAttribute("type") );
        NodeList inputListXML = constructorXML.getElementsByTagName("Input");
        String iconFilename = constructorXML.getAttribute("icon");
        DataType inputs[] = new DataType[ inputListXML.getLength()];
        String names[] = new String[ inputListXML.getLength()];
        for( int j = 0; j < inputs.length; j++ )
        {
          Element input = (Element)inputListXML.item(j);
          inputs[j] = DataType.getDataType( input.getAttribute("type") );
          names[j] = input.getAttribute("name");
        }
        Constructor constructor = new Constructor( constructorName, constructorType, inputs, names, iconFilename );
        addConstructor( constructor );
      }
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }

  }

  public static int getConstructorCount()
  {
    return constructors.size();
  }

  public static Constructor getConstructor( int i )
  {
    return (Constructor)constructors.elementAt( i );
  }

  public static Constructor getConstructor( String name )
  {
    return (Constructor)constructorsHashMap.get(name);
  }


  public String name;
  public DataType constructed;
  public DataType components[];
  public String names[];
  public String iconFilename;

  public Constructor( String name, DataType combined, DataType components[], String names[],  String iconFilename )
  {
    this.name = name;
    this.constructed = combined;
    this.components = components;
    this.names = names;
    this.iconFilename = iconFilename;
  }

  public String getName()
  {
    return name;
  }

  public DataType getType()
  {
    return constructed;
  }

  public int getInputCount()
  {
    return components.length;
  }

  public DataType getInputType( int i )
  {
    return components[i];
  }

  public String getInputName( int i )
  {
    return names[i];
  }

  public String getIconFilename()
  {
    return iconFilename;
  }


  public String toString()
  {
    return name;
  }

}