package vsl.lang;

import java.awt.Color;
import java.awt.image.BufferedImage;

import java.io.File;
import java.io.IOException;

import java.util.HashMap;
import java.util.Vector;

import javax.imageio.ImageIO;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/*
 * Objektreprsentation eines untersttzten Datentyps.
 */
public class DataType
{

  /*
   * Java-Datentypen.
   */
  public static final int J_UNDEFINED = -1;
  public static final int J_BOOLEAN = 0;
  public static final int J_INT = 1;
  public static final int J_LONG = 2;
  public static final int J_FLOAT = 3;
  public static final int J_DOUBLE = 4;

  /*
   * Es wird zwischen Elementar-, Vektor- und Matrix-Datentypen
   * unterschieden.
   */
  public static final int UNKNOWN = -1;
  public static final int ELEMENTAL = 0;
  public static final int VECTOR = 1;
  public static final int MATRIX = 2;

  /*
   * Name des Datentyps
   */
  public String name;

  /*
   * grundlegender Datentyp
   */
  public String baseType;

  /*
   * kompatibler Java-Datentyp
   */
  public int javaType;

  /*
   * Einordnung (Elementar, Vektor oder Matrix)
   */
  public int typeClass;

  /*
   * Zeilen- und Spalten-Zahl
   */
  public int rows, columns;

  /*
   * Darstellung durch Icon
   */
  public BufferedImage icon;

  /*
   * farbliche Unterscheidung
   */
  public Color color;

  /*
   * Default-Wert
   */
  public DataValue defaultValue = null;

  /*
   * Erzeugt einen neuen Datentyp.
   */
  public DataType( String name, int typeClass, String baseType, int jtype, int rows, int columns,
                   String iconFilename, Color color )
  {
    this.name = name;
    this.baseType = baseType;
    this.typeClass = typeClass;
    this.rows = rows;
    this.columns = columns;
    this.javaType = jtype;
    this.color = color;
    try
    {
      icon = ImageIO.read( new File( iconFilename ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }
  }

  /*
   * Setzt den Default-Wert.
   */
  public void setDefaultValue( DataValue defaultValue )
  {
    this.defaultValue = defaultValue;
  }

  /*
   * Gibt den Default-Wert zurck.
   */
  public DataValue getDefaultValue()
  {
    return defaultValue;
  }

  /*
   * Ist ein Default-Wert vorhanden?
   */
  public boolean hasDefaultValue()
  {
    if ( defaultValue == null )
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /*
   * Erzeugt einen Farbwert aus einem String.
   * Format: "r g b" (r,g,b sind zwischen 0 und 255)
   */
  public static Color getColor( String colorString )
  {
    int r, g, b;
    int space1, space2;
    space1 = colorString.indexOf( " " );
    space2 = colorString.lastIndexOf( " " );
    r = Integer.parseInt( colorString.substring( 0, space1 ) );
    g = Integer.parseInt( colorString.substring( space1 + 1, space2 ) );
    b = Integer.parseInt( colorString.substring( space2 + 1 ) );
    return new Color( r, g, b );
  }

  public static int getClass( String baseTypeString )
  {
    if ( baseTypeString.equals( "elemental" ) )
    {
      return ELEMENTAL;
    }
    if ( baseTypeString.equals( "vector" ) )
    {
      return VECTOR;
    }
    if ( baseTypeString.equals( "matrix" ) )
    {
      return MATRIX;
    }
    return -1;
  }

  /*
   * Gibt die Nummer des kompatiblen Java-Datentyps zurck.
   */
  public static int getJavaType( String javaTypeString )
  {
    if ( javaTypeString.equals( "boolean" ) )
    {
      return J_BOOLEAN;
    }
    if ( javaTypeString.equals( "int" ) )
    {
      return J_INT;
    }
    if ( javaTypeString.equals( "float" ) )
    {
      return J_FLOAT;
    }
    if ( javaTypeString.equals( "double" ) )
    {
      return J_DOUBLE;
    }
    return J_UNDEFINED;
  }

  /*
   * Gibt den kompatiblen Java-Typ zurck.
   */
  public int getJavaType()
  {
    return javaType;
  }

  /*
   * Gibt den Basis-Datentyp zurck.
   */
  public String getBaseType()
  {
    return baseType;
  }

  /*
   * Gibt die Typ-Einordnung zurck.
   */
  public int getTypeClass()
  {
    return typeClass;
  }

  /*
   * Gibt den Datentyp als String zurck.
   */
  public String toString()
  {
    return name;
  }

  /*
   * Gibt die Farbe zurck.
   */
  public Color getColor()
  {
    return color;
  }

  /*
   * Gibt das Image des Icons zurck.
   */
  public BufferedImage getIcon()
  {
    return icon;
  }

  public boolean equals( DataType other )
  {
    if ( baseType == other.baseType && rows == other.rows && columns == other.columns )
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  /*
   * Folgend ist ein statischer Teil, der den Zugriff auf die untersttzten
   * Datentypen ermglicht. Diese werden aus einer XML-Datei eingelesen.
   */

  /*
   * Speicherung der Datentypen nach Einordung getrennt
   */
  public static Vector matrix;
  public static Vector vector;
  public static Vector elemental;

  /*
   * Hashmap fr die schnelle Suche nach Namen
   */
  public static HashMap dataTypesHashMap;

  /*
   * Initialierung und Einlesen der Daten
   */
  static
  {
    elemental = new Vector();
    vector = new Vector();
    matrix = new Vector();
    dataTypesHashMap = new HashMap();
    importDataTypes( "xml/datatypes.xml" );
  }

  /*
   * Anzahl der Matrix-Datentypen
   */
  public static int getMatrixDataTypesCount()
  {
    return matrix.size();
  }

  /*
   * Gibt alle Matrix-Datentypen zurck.
   */
  public static Vector getMatrixDataTypes()
  {
    return matrix;
  }

  public static DataType getMatrixDataType( int i )
  {
    return ( DataType ) ( matrix.elementAt( i ) );
  }

  public static DataType getElementalDataType( int i )
  {
    return ( DataType ) ( elemental.elementAt( i ) );
  }

  public static DataType getVectorDataType( int i )
  {
    return ( DataType ) ( vector.elementAt( i ) );
  }

  /*
   * Anzahl der Vektor-Datentypen
   */
  public static int getVectorDataTypesCount()
  {
    return vector.size();
  }

  /*
   * Gibt alle Vektor-Datentypen zurck.
   */
  public static Vector getVectorDataTypes()
  {
    return vector;
  }

  /*
   * Anzahl der Elementar-Datentypen
   */
  public static int getElementalDataTypesCount()
  {
    return elemental.size();
  }

  /*
   * Gibt alle Elementar-Datentypen zurck.
   */
  public static Vector getElementalDataTypes()
  {
    return elemental;
  }

  /*
   * Findet den Datentyp nach Namen und gibt ihn zurck.
   */
  public static DataType getDataType( String name )
  {
    if ( dataTypesHashMap.containsKey( name ) )
    {
      return ( DataType ) dataTypesHashMap.get( name );
    }
    else
    {
      return null;
    }
  }

  /*
   * Einlesen der untersttzten Datentypen aus einer XML-Datei.
   */
  public static void importDataTypes( String filename )
  {
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating( true );

      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler( new DefaultHandler() );

      Document document = builder.parse( filename );

      Element rootElement = document.getDocumentElement();

      NodeList datatypeListXML = rootElement.getElementsByTagName( "DataType" );
      for ( int i = 0; i < datatypeListXML.getLength(); i++ )
      {
        Element operatorXML = ( Element ) ( datatypeListXML.item( i ) );
        String dataTypeName = operatorXML.getAttribute( "name" );
        int typeClass = DataType.getClass( operatorXML.getAttribute( "class" ) );
        String baseType = operatorXML.getAttribute( "btype" );
        int javaType = DataType.getJavaType( operatorXML.getAttribute( "jtype" ) );
        int rows = Integer.parseInt( operatorXML.getAttribute( "rows" ) );
        int columns = Integer.parseInt( operatorXML.getAttribute( "columns" ) );
        String dataTypeClass = operatorXML.getAttribute( "class" );
        String iconFilename = operatorXML.getAttribute( "icon" );
        Color color = DataType.getColor( operatorXML.getAttribute( "color" ) );
        DataType newType = new DataType( dataTypeName, typeClass, baseType, javaType, rows,
                                         columns, iconFilename, color );
        switch ( newType.getTypeClass() )
        {
          case DataType.ELEMENTAL:
            elemental.addElement( newType );
            break;
          case DataType.VECTOR:
            vector.addElement( newType );
            break;
          case DataType.MATRIX:
            matrix.addElement( newType );
            break;
        }
        dataTypesHashMap.put( newType.toString(), newType );
        if ( newType.getJavaType() != DataType.J_UNDEFINED )
        {
          String defaultValue = operatorXML.getAttribute( "default" );
          DataValue dataObject = new DataValue( newType, defaultValue );
          newType.setDefaultValue( dataObject );
        }
      }
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }
  }

}