package vsl.lang;

import java.net.URL;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import vsl.lang.*;

/*
 * Klasse, die Zugriff auf ein Datenflussdiagramm gewhrt.
 */
public class Diagram
{

  /*
   * DOM-Dokument und Wurzel des Diagramms.
   */
  protected Document diagramDocument;
  protected Element diagramRoot;

  private NodeList inputList = null;
  private NodeList outputList = null;

  protected String name;
  protected String type;

  protected String category;

  public Diagram( Document diagramDocument )
  {
    this.diagramDocument = diagramDocument;
    init();
  }

  public Diagram( String documentFileName ) throws
      SAXException, SAXParseException, IOException
  {

    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating( true );

      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler( new DefaultHandler() );

      diagramDocument = builder.parse( documentFileName );

      init();
    }
    catch ( ParserConfigurationException e )
    {
      System.err.println( e );
    }

  }

  public String toString()
  {
    return getName();
  }

  private void init()
  {
    diagramRoot = diagramDocument.getDocumentElement();
    name = diagramRoot.getAttribute( "name" );
    inputList = diagramDocument.getDocumentElement().getElementsByTagName( "Input" );
    outputList = diagramDocument.getDocumentElement().getElementsByTagName( "Output" );
  }

  public Document getDocument()
  {
    return diagramDocument;
  }

  public String getName()
  {
    return name;
  }

  public String getCategory()
  {
    return category;
  }

  public int getCategoryInt()
  {
    return Diagram.getCategory(category);
  }

  public int getInputCount()
  {
    return inputList.getLength();
  }

  public String getInputName( int i )
  {
    Element e = ( Element ) inputList.item( i );
    return e.getAttribute( "name" );
  }

  public String getInputType( int i )
  {
    Element e = ( Element ) inputList.item( i );
    return e.getAttribute( "type" );
  }

  public String getInputDefault( int i )
  {
    Element e = ( Element ) inputList.item( i );
    return e.getAttribute( "default" );
  }

  public int getOutputCount()
  {
    return outputList.getLength();
  }

  public String getOutputName( int i )
  {
    Element e = ( Element ) outputList.item( i );
    return e.getAttribute( "name" );
  }

  public String getOutputType( int i )
  {
    Element e = ( Element ) outputList.item( i );
    return e.getAttribute( "type" );
  }

  public void setCategory( String category )
  {
    this.category = category;
  }

  public static final int UNDEFINED = -1;
  public static final int NEW = 0;
  public static final int LIGHTING = 1;
  public static final int TRANSFORMATION = 2;
  public static final int TEXTURE = 3;
  public static final int MISC = 4;

  public static String catStrings[] = { "New", "Lighting", "Transformation", "Texture", "Misc" };

  public static final int catCount = 5;

  public static Vector diagrams[];

  public static HashMap diagramsHashMap;

  static
  {
    diagrams = new Vector[catCount];
    for ( int i = 0; i < catCount; i++ )
    {
      diagrams[i] = new Vector();
    }
    diagramsHashMap = new HashMap();
    importDiagram( "xml/diagrams.xml" );
  }

  public static int getCategory( String catString )
  {
    for( int i =0 ; i< catCount; i ++ )
    {
      if( catStrings[i].equals( catString ) )
      {
        return i;
      }
    }
    return UNDEFINED;
  }

  public static String getCategory( int category )
  {
    return catStrings[category];
  }


  public static void importDiagram( String filename )
  {
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating( true );

      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler( new DefaultHandler() );

      Document document = builder.parse( filename );

      Element rootElement = document.getDocumentElement();

      NodeList diagramListXML = rootElement.getElementsByTagName( "Diagram" );
      for ( int i = 0; i < diagramListXML.getLength(); i++ )
      {
        Element diagramXML = ( Element ) ( diagramListXML.item( i ) );
        Diagram diagram = new Diagram( diagramXML.getAttribute( "filename" ) );
        diagram.setCategory( diagramXML.getAttribute("category") );
        addDiagram( diagram );
      }
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }
  }

  public static void addDiagram( Diagram diagram )
  {
    String diagramCategory = diagram.getCategory() ;
    int diagramCategoryI = diagram.getCategory( diagramCategory );
    diagrams[ diagramCategoryI ].add(diagram);
    String diagramName = diagram.getName();
    diagramsHashMap.put(diagramName, diagram );
  }

  public static Diagram getDiagram( String name )
  {
    if( diagramsHashMap.containsKey(name) )
    {
      return ( Diagram ) diagramsHashMap.get( name );
    }
    else return null;
  }

  public static Vector getDiagram( int category )
  {
    return diagrams[category];
  }

}
