package vsl.lang;

import java.awt.image.BufferedImage;

import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import java.net.URL;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;

import java.util.HashMap;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/*
 * Objektreprsentation einer Funktion
 */
public class Function
{

  /*
   * Funktionen werden in 3 Kategorien eingeteilt. Es gibt die
   * Bereiche Mathematik, Geometrie und Textur.
   */
  public static final int CG_MATHEMATICAL = 0;
  public static final int CG_GEOMETRIC = 1;
  public static final int CG_TEXTURE = 2;

  /*
   * Name der Funktion
   */
  public String name;

  /*
   * Kategorie
   */
  public int functionClass;

  /*
   * Icon zur Darstellung
   */
  public String iconFilename;
  public BufferedImage icon;

  /*
   * Ein-eindeutiger Schlssel zur Unterscheidung der Funktionen.
   * Funktionen knnen den gleichen Namen, aber unterschiedliche
   * Parameter haben.
   */
  public String key;

  /*
   * Rckgabewerte der Funktion
   */
  public DataType returnType;

  /*
   * Parameterliste
   */
  public Parameter parameterList[];

  /*
   * Erzeugt eine neue Funktion.
   */
  public Function( String name, DataType returnType, Parameter parameterList[],
                   String functionClass, String iconFilename )
  {
    this.name = name;
    this.iconFilename = iconFilename;
    this.returnType = returnType;
    this.parameterList = parameterList;
    try
    {
      icon = ImageIO.read( new File( iconFilename ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }

    if ( functionClass.equals( "mathematical" ) )
    {
      this.functionClass = CG_MATHEMATICAL;
    }
    else if ( functionClass.equals( "geometric" ) )
    {
      this.functionClass = CG_GEOMETRIC;
    }
    else if ( functionClass.equals( "texture" ) )
    {
      this.functionClass = CG_TEXTURE;
    }
    key = name;
    for ( int i = 0; i < parameterList.length; i++ )
    {
      key += parameterList[i].getType();
    }
  }

  /*
   * Gibt die Kategorie zurck.
   */
  public int getFunctionClass()
  {
    return functionClass;
  }

  /*
   * Gibt den Namen zurck.
   */
  public String getName()
  {
    return name;
  }

  /*
   * Gibt den Schlssel zurck.
   */
  public String getKey()
  {
    return key;
  }

  /*
   * Gibt die Anzahl der Parameter zurck.
   */
  public int getParameterCount()
  {
    return parameterList.length;
  }

  /*
   * Gibt Parameter i zurck.
   */
  public Parameter getParameter( int i )
  {
    return parameterList[i];
  }

  /*
   * Gibt den Rckgabewert der Funktion zurck.
   */
  public DataType getReturnType()
  {
    return returnType;
  }

  /*
   * Gibt das Icon zurck.
   */
  public BufferedImage getIcon()
  {
    return icon;
  }

  /*
   * Stringdarstellung
   */
  public String toString()
  {
    String returnTypeString = returnType.toString();
    String string = returnTypeString + " " + name + "(";

    for ( int j = 0; j < parameterList.length; j++ )
    {
      if ( j != 0 )
      {
        string += ", ";
      }
      string += parameterList[j].getType().toString() + " " + parameterList[j].getName();
    }
    string += ")";
    return string;
  }

  /*
   * Folgend ist ein statischer Teil, der den Zugriff auf untersttzte
   * Funktionen ermglicht.
   */

  /*
   * Funktionen werden nach den Kategorien getrennt gepeichert.
   */
  public static Vector mathematical;
  public static Vector geometric;
  public static Vector texture;

  /*
   * Hashmap zur schnellen Suche nach Schlssel-String.
   */
  public static HashMap functionsHashMap;

  /*
   * Initialisierung und ein Einlesen
   */
  static
  {
    mathematical = new Vector();
    geometric = new Vector();
    texture = new Vector();
    functionsHashMap = new HashMap();
    importFunctions( "xml/functions.xml" );
  }

  /*
   * Funktion hinzufgen
   */
  public static void addFunction( Function function )
  {
    switch ( function.getFunctionClass() )
    {
      case Function.CG_MATHEMATICAL:
        mathematical.add( function );
        break;
      case Function.CG_GEOMETRIC:
        geometric.add( function );
        break;
      case Function.CG_TEXTURE:
        texture.add( function );
        break;
    }
    functionsHashMap.put( function.getKey(), function );
  }

  /*
   * Gibt die Funktion des entsprechenden Schssels zurck.
   */
  public static Function getFunction( String key )
  {
    return ( Function ) functionsHashMap.get( key );
  }

  /*
   * Gibt alle mathematischen Funktionen zurck.
   */
  public static Vector getMathFunctions()
  {
    return mathematical;
  }

  /*
   * Gibt alle geometrischen Funktionen zurck.
   */
  public static Vector getGeomFunctions()
  {
    return geometric;
  }

  /*
   * Gibt alle Textur-Funktionen zurck.
   */
  public static Vector getTexFunctions()
  {
    return texture;
  }

  /*
   * Anzahl der mathematischen Funktionen
   */
  public static int getMathFunctionCount()
  {
    return mathematical.size();
  }

  /*
   * Gibt mathematischen Funktion i zurck.
   */
  public static Function getMathFunction( int i )
  {
    return ( Function ) mathematical.elementAt( i );
  }

  /*
   * Anzahl der geometrischen Funktionen
   */
  public static int getGeomFunctionCount()
  {
    return geometric.size();
  }

  /*
   * Gibt geometrische Funktion i zurck.
   */
  public static Function getGeomFunction( int i )
  {
    return ( Function ) geometric.elementAt( i );
  }

  /*
   * Anzahl der Textur-Funktionen
   */
  public static int getTexFunctionCount()
  {
    return texture.size();
  }

  /*
   * Gibt Textur-Funktion i zurck.
   */
  public static Function getTexFunction( int i )
  {
    return ( Function ) texture.elementAt( i );
  }

  /*
   * Funktionen aus XML-Datei einlesen
   */
  public static void importFunctions( String filename )
  {
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating( true );

      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler( new DefaultHandler() );

      Document document = builder.parse( filename );

      Element rootElement = document.getDocumentElement();

      NodeList functionsXML = rootElement.getElementsByTagName( "Function" );
      for ( int i = 0; i < functionsXML.getLength(); i++ )
      {
        Element functionXML = ( Element ) ( functionsXML.item( i ) );
        String functionName = functionXML.getAttribute( "name" );
        String functionClass = functionXML.getAttribute( "class" );
        String iconFilename = functionXML.getAttribute( "icon" );

        NodeList parametersListXML = functionXML.getElementsByTagName( "Parameters" );
        for ( int j = 0; j < parametersListXML.getLength(); j++ )
        {
          Element parametersXML = ( Element ) parametersListXML.item( j );

          NodeList returnListXML = parametersXML.getElementsByTagName( "Return" );
          Element returnXML = ( Element ) ( returnListXML.item( 0 ) );

          DataType returnType = DataType.getDataType( returnXML.getAttribute( "type" ) );

          NodeList parameterListXML = parametersXML.getElementsByTagName( "Parameter" );
          Parameter[] parameterList = new Parameter[parameterListXML.getLength()];
          for ( int k = 0; k < parameterListXML.getLength(); k++ )
          {
            Element parameterXML = ( Element ) ( parameterListXML.item( k ) );
            Parameter parameter = new Parameter( parameterXML.getAttribute( "name" ),
                                                 DataType.getDataType( parameterXML.getAttribute(
                "type" ) ) );
            parameterList[k] = parameter;
          }
          Function function = new Function( functionName, returnType, parameterList, functionClass,
                                            iconFilename );
          addFunction( function );
        }
      }

    }
    catch ( Exception e )
    {
      System.err.println( e );
    }

  }

}