package vsl.lang;

import java.awt.image.BufferedImage;

import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import java.util.Vector;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/*
 * Objektreprsentation eines Operators
 */
public class Operator
{

  /*
   * Es wird zwischen Infix-, Prfix- und Postfix-Operatoren
   * unterschieden.
   */
  public static final int INFIX = 1;
  public static final int PRAEFIX = 2;
  public static final int POSTFIX = 3;

  /*
   * Operatortyp (infix, prfix, postfix)
   */
  public int operatorType;

  /*
   * Name des Operators
   */
  public String name;

  /*
   * Operator-Symbol
   */
  public String symbol;

  /*
   * Ergebnistyp
   */
  public DataType result;

  /*
   * Operanden
   */
  public DataType operand1;
  public DataType operand2;

  /*
   * Ein-eindeutiger Schlssel
   */
  public String key;

  /*
   * Icon
   */
  public BufferedImage icon;

  /*
   * Erzeugt einen neuen Operator
   */
  public Operator( String name, String symbol, DataType result, DataType operand1,
                   DataType operand2, int operatorType, String iconFilename )
  {
    this.name = name;
    this.symbol = symbol;
    this.result = result;
    this.operand1 = operand1;
    this.operand2 = operand2;
    this.operatorType = operatorType;
    try
    {
      icon = ImageIO.read( new File( iconFilename ) );
    }
    catch ( IOException e )
    {
      System.err.println( e );
    }
    switch ( operatorType )
    {
      case Operator.INFIX:
        this.key = operand1.toString() + name + operand2.toString();
        break;
      case Operator.POSTFIX:
        this.key = name + operand1.toString();
        break;
      case Operator.PRAEFIX:
        this.key = operand1.toString() + name;
    }
  }

  /*
   * Gibt den Schlssel zurck.
   */
  public String getKey()
  {
    return key;
  }

  /*
   * Gibt den Operatornamen zurck.
   */
  public String getName()
  {
    return name;
  }

  /*
   * Gibt Operator-Symbol zurck.
   */
  public String getSymbol()
  {
    return symbol;
  }

  /*
   * Gibt Ergebnistyp zurck.
   */
  public DataType getResult()
  {
    return result;
  }

  /*
   * Gibt 1. Operanden zurck.
   */
  public DataType getOperand1()
  {
    return operand1;
  }

  /*
   * Gibt 2. Operanden zurck.
   */
  public DataType getOperand2()
  {
    return operand2;
  }

  /*
   * Gibt Operanden zurck.
   */
  public int getOperatorType()
  {
    return operatorType;
  }

  public BufferedImage getIcon()
  {
    return icon;
  }

  public String toString()
  {

    String string = null;
    try
    {
      if ( operatorType == INFIX )
      {
        string = getResult().toString() + " = " + getOperand1().toString() + " " +
            getSymbol() + " " + getOperand2().toString();
      }
      if ( operatorType == PRAEFIX )
      {
        string = getSymbol() + getOperand1().toString();
      }
      if ( operatorType == POSTFIX )
      {
        string = getOperand1().toString() + getSymbol();
      }
    }
    catch ( Exception e )
    {
      System.out.println( e );
    }
    return string;
  }

  /*
   * Folgend ist ein statischer Teil, der den Zugriff auf die untersttzten
   * Operatoren ermglicht. Diese werden aus einer XML-Datei eingelesen.
   */

  /*
   * Speicherung der Operatoren nach Typ getrennt
   */
  public static Vector praefix;
  public static Vector postfix;
  public static Vector infix;

  /*
   * Hashmap fr die schnelle Suche nach Schlssel
   */
  public static HashMap operatorsHashMap;

  /*
   * Initialierung und Einlesen der Daten
   */
  static
  {
    praefix = new Vector();
    postfix = new Vector();
    infix = new Vector();
    operatorsHashMap = new HashMap();
    importOperators( "xml/operators.xml" );
  }

  /*
   * Gibt alle Infix-Operatoren zurck.
   */
  public static Vector getInfixOperators()
  {
    return infix;
  }

  /*
   * Gibt alle Prfix-Operatoren zurck.
   */
  public static Vector getPraefixOperators()
  {
    return praefix;
  }

  /*
   * Gibt alle Postfix-Operatoren zurck.
   */
  public static Vector getPostfixOperators()
  {
    return postfix;
  }

  /*
   * Anzahl der Infix-Operatoren
   */
  public static int getInfixOperatorCount()
  {
    return infix.size();
  }

  /*
   * Gibt Prfix-Operator i zurck.
   */
  public static Operator getPraefixOperator( int i )
  {
    return ( Operator ) praefix.elementAt( i );
  }

  /*
   * Anzahl der Infix-Operatoren
   */
  public static int getPraefixOperatorCount()
  {
    return praefix.size();
  }

  /*
   * Gibt Postfix-Operator i zurck.
   */
  public static Operator getPostfixOperator( int i )
  {
    return ( Operator ) postfix.elementAt( i );
  }

  /*
   * Anzahl der Postfix-Operatoren
   */
  public static int getPostfixOperatorCount()
  {
    return postfix.size();
  }

  /*
   * Gibt Infix-Operator i zurck.
   */
  public static Operator getInfixOperator( int i )
  {
    return ( Operator ) infix.elementAt( i );
  }

  /*
   * Fgt einen Operator hinzu.
   */
  public static void addOperator( Operator operator )
  {
    switch ( operator.getOperatorType() )
    {
      case Operator.INFIX:
        infix.addElement( operator );
        break;
      case Operator.PRAEFIX:
        praefix.addElement( operator );
        break;
      case Operator.POSTFIX:
        postfix.addElement( operator );
        break;
    }
    operatorsHashMap.put( operator.getKey(), operator );
  }

  /*
   * Gibt Postfix-Operator mit Schlssel key zurck.
   */
  public static Operator getOperator( String key )
  {
    return ( Operator ) ( operatorsHashMap.get( key ) );
  }


  /*
   * Gibt den Operatortyp (infix, prfix, postfix) zum String
   * zurck.
   */
  public static int getOperatorType( String operatorTypeS )
  {
    if ( operatorTypeS.equals( "infix" ) )
    {
      return Operator.INFIX;
    }
    if ( operatorTypeS.equals( "praefix" ) )
    {
      return Operator.PRAEFIX;
    }
    if ( operatorTypeS.equals( "postfix" ) )
    {
      return Operator.POSTFIX;
    }
    return -1;
  }

  /*
   * Einlesen der verfgbaren Operatoren aus einer XML-Datei.
   */
  public static void importOperators( String filename )
  {
    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setValidating( true );

      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler( new DefaultHandler() );

      Document document = builder.parse( filename );

      Element rootElement = document.getDocumentElement();

      NodeList operatorListXML = rootElement.getElementsByTagName( "Operator" );
      for ( int i = 0; i < operatorListXML.getLength(); i++ )
      {
        Element operatorXML = ( Element ) ( operatorListXML.item( i ) );
        String operatorName = operatorXML.getAttribute( "name" );
        String operatorTypeS = operatorXML.getAttribute( "type" );
        String iconFilename = operatorXML.getAttribute( "icon" );
        int operatorType = 0;
        operatorType = Operator.getOperatorType( operatorTypeS );
        String operatorSymbol = operatorXML.getAttribute( "symbol" );

        NodeList parametersListXML = operatorXML.getElementsByTagName( "Parameters" );
        for ( int j = 0; j < parametersListXML.getLength(); j++ )
        {
          Element parametersXML = ( Element ) parametersListXML.item( j );

          NodeList returnListXML = parametersXML.getElementsByTagName( "Result" );
          Element returnXML = ( Element ) ( returnListXML.item( 0 ) );

          DataType resultType = DataType.getDataType( returnXML.getAttribute( "type" ) );

          NodeList operandListXML = parametersXML.getElementsByTagName( "Operand" );
          Element operand1XML = ( Element ) ( operandListXML.item( 0 ) );
          DataType operand1 = DataType.getDataType( operand1XML.getAttribute( "type" ) );
          DataType operand2 = null;
          if ( operandListXML.getLength() == 2 )
          {
            Element operand2XML = ( Element ) ( operandListXML.item( 1 ) );
            operand2 = DataType.getDataType( operand2XML.getAttribute( "type" ) );
          }
          Operator operator = new Operator( operatorName, operatorSymbol, resultType, operand1,
                                            operand2, operatorType, iconFilename );
          addOperator( operator );
        }
      }

    }
    catch ( Exception e )
    {
      System.err.println( e );
    }

  }

}