package vsl.util;

import java.io.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

import org.w3c.dom.*;

/*
 * XSLT-Prozessor zur Transformation von Shadern im DOM-Format
 */
public class ShaderTransformer
{

  private Transformer xslTransformer;
  private Transformer fileTransformer;

  private Result result;
  private Source xslSource;

  private StringWriter stringWriter;

  public ShaderTransformer( String xslFile )
  {
    xslSource = new StreamSource( xslFile );
    init();
  }

  public void init()
  {
    try
    {
      xslTransformer = TransformerFactory.newInstance().newTransformer( xslSource );
      fileTransformer = TransformerFactory.newInstance().newTransformer();
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }
  }

  public void setXSLFile( String filename )
  {
    xslSource = new StreamSource( filename );
    try
    {
      xslTransformer = TransformerFactory.newInstance().newTransformer( xslSource );
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }
  }

  public void saveFile( Document document, String filename )
  {
    try
    {
      Source source = new DOMSource( document.getDocumentElement() );
      Result result = new StreamResult( new FileWriter( filename ) );
      fileTransformer.transform( source, result );
    }
    catch( Exception e )
    {
      System.err.println(e);
    }
  }

  public void saveFile( Document document, File file )
  {
    try
    {
      Source source = new DOMSource( document.getDocumentElement() );
      Result result = new StreamResult( new FileWriter( file ) );
      fileTransformer.transform( source, result );
    }
    catch( Exception e )
    {
      System.err.println(e);
    }
  }

  public void transformToFile( Document document, File file )
  {
    try
    {
      Source source = new DOMSource( document.getDocumentElement() );
      result = new StreamResult( new FileWriter( file ) );
      xslTransformer.transform( source, result );
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }
  }


  public String transform( Document document )
  {
    stringWriter = new StringWriter();
    result = new StreamResult( stringWriter );
    Source source = new DOMSource( document.getDocumentElement() );
    try
    {
      xslTransformer.transform( source, result );
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }
    return stringWriter.toString();
  }

  public String transform( String filename )
  {
    stringWriter = new StringWriter();
    result = new StreamResult( stringWriter );
    Source source = new StreamSource( new File( filename ) );
    try
    {
      xslTransformer.transform( source, result );
    }
    catch ( Exception e )
    {
      System.err.println( e );
    }
    return stringWriter.toString();
  }

}